require 'test_helper'

require 'wsdl_mapper/dom_parsing/parser'
require 'wsdl_mapper/generation/context'
require 'wsdl_mapper/dom_generation/documented_schema_generator'

module DomGenerationTests
  module GeneratorTests
    class DocumentedSchemaGeneratorTest < GenerationTest
      include WsdlMapper::DomGeneration

      def generate(name)
        schema = TestHelper.parse_schema name
        generator = DocumentedSchemaGenerator.new context
        generator.generate schema
      end

      def test_class_documentation
        generate 'basic_note_type.xsd'

        assert_file_is 'note_type.rb', <<RUBY
# @xml_name noteType
class NoteType
  # @!attribute to
  #   @return [String]
  #   @xml_name to
  attr_accessor :to

  # @!attribute from
  #   @return [String]
  #   @xml_name from
  attr_accessor :from

  # @!attribute heading
  #   @return [String]
  #   @xml_name heading
  attr_accessor :heading

  # @!attribute body
  #   @return [String]
  #   @xml_name body
  attr_accessor :body

  # This is the autogenerated default constructor.
  #
  # @param to [String]
  # @param from [String]
  # @param heading [String]
  # @param body [String]
  #
  def initialize(to: nil, from: nil, heading: nil, body: nil)
    @to = to
    @from = from
    @heading = heading
    @body = body
  end
end
RUBY

        assert_file_is '.yardopts', <<SH
--tag xml_name:"XML Name"
--tag xml_namespace:"XML Namespace"
--tag xml_value:"XML Value"
--tag soap_name:"SOAP Name"
--markup markdown
--markup-provider kramdown
**/*.rb
SH
      end
    end
  end
end

